package com.chelvc.framework.database.support;

import java.util.Map;

import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.core.toolkit.support.SerializedLambda;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.apache.ibatis.reflection.property.PropertyNamer;

/**
 * MyBatis工具类
 *
 * @author Woody
 * @date 2025/3/14
 */
public final class MyBatisUtils {
    /**
     * 表名称/对象映射表
     */
    private static final Map<String, Table> TABLE_NAME_MAPPING = Maps.newConcurrentMap();

    private MyBatisUtils() {
    }

    /**
     * 字符串参数特殊字符转义
     *
     * @param value 参数值
     * @return 参数值
     */
    public static String escape(String value) {
        if (StringUtils.isEmpty(value)) {
            return value;
        }

        // 判断参数是否需要转义
        boolean needs = false;
        int size = value.length();
        for (int i = 0; i < size; i++) {
            char c = value.charAt(i);
            if (c == '\u0000' || c == '\n' || c == '\r' || c == '\u001a' || c == '"' || c == '\'' || c == '\\') {
                needs = true;
                break;
            }
        }
        if (!needs) {
            return value;
        }

        // 特殊字符转义
        StringBuilder buffer = new StringBuilder((int) ((double) size * 1.1D));
        for (int i = 0; i < size; i++) {
            char c = value.charAt(i);
            switch (c) {
                case '\u0000':
                    buffer.append('\\');
                    buffer.append('0');
                    break;
                case '\n':
                    buffer.append('\\');
                    buffer.append('n');
                    break;
                case '\r':
                    buffer.append('\\');
                    buffer.append('r');
                    break;
                case '\u001a':
                    buffer.append('\\');
                    buffer.append('Z');
                    break;
                case '\'':
                    buffer.append('\'');
                    buffer.append('\'');
                    break;
                case '\\':
                    buffer.append('\\');
                    buffer.append('\\');
                    break;
                default:
                    buffer.append(c);
            }
        }
        return buffer.toString();
    }

    /**
     * 去掉表、字段名称反引号
     *
     * @param name 表、字段名称
     * @return 表、字段名称
     */
    public static String unquote(String name) {
        return isQuoted(name) ? name.substring(1, name.length() - 1) : name;
    }

    /**
     * 判断表、字段名称是否包含反引号
     *
     * @param name 表、字段名称
     * @return true/false
     */
    public static boolean isQuoted(String name) {
        return StringUtils.notEmpty(name) && name.charAt(0) == '`' && name.charAt(name.length() - 1) == '`';
    }

    /**
     * 将Getter方法转换成字段名称
     *
     * @param getter Getter方法
     * @return 字段名称
     */
    public static String getter2column(@NonNull SFunction<?, ?> getter) {
        SerializedLambda lambda = LambdaUtils.resolve(getter);
        String property = PropertyNamer.methodToProperty(lambda.getImplMethodName());
        return property2column(lambda.getInstantiatedType(), property);
    }

    /**
     * 将Getter方法转换成属性名称
     *
     * @param getter Getter方法
     * @return 属性名称
     */
    public static String getter2property(@NonNull SFunction<?, ?> getter) {
        SerializedLambda lambda = LambdaUtils.resolve(getter);
        return PropertyNamer.methodToProperty(lambda.getImplMethodName());
    }

    /**
     * 将属性名转换成字段名称
     *
     * @param clazz    实体对象
     * @param property 属性名称
     * @return 字段名称
     */
    public static String property2column(@NonNull Class<?> clazz, @NonNull String property) {
        Map<String, ColumnCache> columns = LambdaUtils.getColumnMap(clazz);
        ColumnCache column = ObjectUtils.ifNull(columns, cs -> cs.get(LambdaUtils.formatKey(property)));
        AssertUtils.nonnull(column, () -> "Column not found for property: " + clazz.getName() + "." + property);
        return column.getColumn();
    }

    /**
     * 获取表上下文
     *
     * @param name 表名称
     * @return 表上下文
     */
    public static Table getTable(@NonNull String name) {
        String unquote = MyBatisUtils.unquote(name);
        Table table = TABLE_NAME_MAPPING.get(unquote);
        if (table == null) {
            table = TABLE_NAME_MAPPING.computeIfAbsent(unquote, k -> {
                TableInfo info = TableInfoHelper.getTableInfo(name);
                if (info == null && MyBatisUtils.isQuoted(name)) {
                    info = TableInfoHelper.getTableInfo(unquote);
                }
                return ObjectUtils.ifNull(info, Table::new);
            });
        }
        return table;
    }

    /**
     * 获取表数据模型
     *
     * @param table 表名
     * @return 数据模型对象
     */
    public static Class<?> getTableModel(@NonNull String table) {
        return ObjectUtils.ifNull(getTable(table), Table::getModel);
    }

    /**
     * 获取表字段上下文
     *
     * @param table 表名称
     * @param field 字段名称
     * @return 表字段上下文
     */
    public static TableField getTableField(@NonNull String table, @NonNull String field) {
        return ObjectUtils.ifNull(getTable(table), context -> context.getField(field));
    }
}
